<?php
/*
   LOGFILE
   Eine Klasse zur Zhlung der Zugriffe auf eine Datei
   Dazu werden Logdatein angelegt, welche dann ausgewertet werden knnen.
*/

// Verzeichnis in dem die LOD-Daten gespeichert werden
define("LOGVERZEICHNIS","podbloggerData/stats/");

class LogFile {

  // ID des zu loggenden Objektes
  var $file_id;

  // Name des zu loggenden Objektes
  var $file_name;

  // Array den Logdaten der letzten zwei Monate
  // Jeder Zugriff wird als ArrayPaar gespeichert (DETAILS)
  var $array_actualLogs;

  // Array den Logdaten lter als zwei Monate
  // Die alten Zugriffe sind zusammengefasst, um Speicherplatz zu sparen
  var $array_oldLogs;

  /*
  * KONSTRUKTOR - Erstellt ein LOGFILE-Objekt
  * ... ld Daten aus zugehriger Logdatei (wenn bereits vorhanden)
  */
  function LogFile($inputfile)
  {
    // Falls der Dateiname einen Verzeichnispfad enthlt diesen zum Teil des Namens machen
    $this->file_id = str_replace("/","___v___",$inputfile);

    // Weitere Variablen initialisieren
    $this->file_name = $inputfile;
    $this->array_actualLogs = array();
    $this->array_oldLogs = array();

    // Daten als LogDatei laden
    if (file_exists(LOGVERZEICHNIS.$this->file_id.".log"))
    {
      $line_array = file(LOGVERZEICHNIS.$this->file_id.".log");
      $lineCount = 1;

      foreach ($line_array as $line)
      {
        $line = chop($line);
        if ($lineCount == 1)
        {
          // In der ersten Zeile stehen die Zugriffe lter als zweiMonate komprimiert
          // daher Sonderbehandlung der ersten Zeile
          $this->array_oldLogs = unserialize($line);
        }
        else
        {
          // Die weiteren Zeilen beinhalten die Zugriffe der letzten zwei Monate
          // pro Zugriff eine Zeile
          $single_log = unserialize($line);
          if (@isset($single_log['UnixTime']))
          {
            // Wenn der logeintrag gltig ist -> in $array_actualLogs aufnehmen
            $this->array_actualLogs[$single_log['UnixTime']] = $single_log;
          }
        }
        $lineCount++;
      } // endforeach

    } // endif file_exists
  }

  function incOldLogLine($type = "ALL")
  {
    if (!isset($this->array_oldLogs[$type]))
    { $this->array_oldLogs[$type] = 0; }
    $this->array_oldLogs[$type]++;
  }

  function getOldLogLineArray()
  {
    return $this->array_oldLogs;
  }

  // Gibt die Anzahl aller Zugriffe auf diese Datei aus
  // $type : ermglicht, dass nur Zugriffe vom Wert $type gezhlt werden
  //	     z.B. $type = "FLASHPLAYER" zhlt nur die Zugriffe, die mit dem Paramerer "FLASHPLAYER" geloggt wurden
  //		 ALL -> Alle Zugriffe
  function zugriffeInsgesamt($type = "ALL")
  {
    $zugriffe = 0;

    // Alte Zugriffe zhlen
    if (isset($this->array_oldLogs[$type]))
    {
      $zugriffe = $zugriffe +  $this->array_oldLogs[$type];
    }

    // Aktuelle Zugriffe zhlen
    if ($type == "ALL")
    {
      // Alle Zugriffe zhlen
      // hier einfach die Eintrge in $this->array_actualLogs zhlen

      $zugriffe = $zugriffe + count($this->array_actualLogs);

    }
    else
    {
      // Nur Zugriffe von einem bestimmten Typ zhlen
      // dazu muss alle Eintrge durchlaufen und geprft werden

      foreach ($this->array_actualLogs as $log)
      {
        if (isset($log['Type']))
        {
          if ($log['Type'] == $type) {$zugriffe++;}
        }
      }

    }

    return $zugriffe;
  }

  // Gibt die Anzahl aller Zugriffe auf diese Datei im Zeitfenster von $startTime bis $endTime
  // $startTime	: (int) UnixTime; Start des Zeitfensters (maximal 2 Monate zurck)
  // $endTime	: (int) UnixTime; Ende des Zeitfensters
  // $type 		: ermglicht, dass nur Zugriffe vom Wert $type gezhlt werden
  //	     	z.B. $type = "FLASHPLAYER" zhlt nur die Zugriffe, die mit dem Paramerer "FLASHPLAYER" geloggt wurden
  //			ALL -> Alle Zugriffe
  function zugriffeVonTimeBisTime($startTime, $endTime, $type)
  {
    $zugriffe = 0;

    // Es werden nur die Zugriffe aus $array_actualLogs ausgewertet
    if ($endTime > time()) {$endTime = time();}

    foreach ($this->array_actualLogs as $log)
    {
      if ( ($log['UnixTime'] >= $startTime) && ($log['UnixTime'] <= $endTime) )
      {
        // Nur Logeintrge werten, die im gegebenen Zeitfenster liegen

        if ($type != "ALL")
        {
          // Logeintrag auf Typ testen
          if (isset($log['Type']))
          {
            if ($log['Type'] == $type) {$zugriffe++;}
          }
        }
        else
        {
          // Logeintrag ohne Typ-Prfung zhlen
          $zugriffe++;
        }

      } // endif Zeitfenstercheck
    }  // endforeach

    return $zugriffe;
  } // endfunction


  // ein Array mit den Loglines im Zeitfenster von $startTime bis $endTime zurck
  // $startTime	: (int) UnixTime; Start des Zeitfensters (maximal 2 Monate zurck)
  // $endTime	: (int) UnixTime; Ende des Zeitfensters
  function loglinesVonTimeBisTime($startTime, $endTime)
  {
    $erg_array = array();

    // Es werden nur die Zugriffe aus $array_actualLogs ausgewertet
    if ($endTime > time()) {$endTime = time();}

    foreach ($this->array_actualLogs as $log)
    {
      if ( ($log['UnixTime'] >= $startTime) && ($log['UnixTime'] <= $endTime) )
      {
        // Nur Logeintrag liegt im Zeitfenster
        array_push($erg_array,$log);
      } // endif Zeitfenstercheck
    }  // endforeach

    return $erg_array;
  } // endfunction


  // Liefert die Anzahl der Zugriffe Heute
  function zugriffeHeute($type = "ALL")
  {
    // Unixzeit vom Beginn des heutigen Tages berechnen
    $day 	= date("j",time());
    $month	= date("n",time());
    $year	= date("Y",time());

    $Tagesbeginn = mktime(0,0,0,$month,$day,$year);

    return $this->zugriffeVonTimeBisTime($Tagesbeginn, time(), $type);
  }

  // Liefert die Anzahl der Zugriffe Gestern
  function zugriffeGestern($type = "ALL")
  {
    // Unixzeit vom Beginn des heutigen Tages berechnen = Ende des Zeitfensters
    $day 	= date("j",time());
    $month	= date("n",time());
    $year	= date("Y",time());

    $GesternEnde = mktime(0,0,0,$month,$day,$year);

    // Unixzeit vom Beginn des gestrigen Tages berechnen
    $GesternAnfang = $GesternEnde - (24 * 60 * 60);

    return $this->zugriffeVonTimeBisTime($GesternAnfang, $GesternEnde, $type);
  }

  // Liefert die Anzahl der Zugriffe diesen Monats
  function zugriffeDieserMonat($type = "ALL")
  {
    // Unixzeit vom Beginn des aktuellen Monats berechnen
    $month	= date("n",time());
    $year	= date("Y",time());

    $Tagesbeginn = mktime(0,0,0,$month,1,$year);

    return $this->zugriffeVonTimeBisTime($Tagesbeginn, time(), $type);
  }

  // Liefert die Anzahl der Zugriffe letzter Monat
  function zugriffeLetzterMonat($type = "ALL")
  {
    // Unixzeit vom Beginn des aktuellen Monats berechnen = Ende des Zeitfensters
    $month	= date("n",time());
    $year	= date("Y",time());

    $ZeitfensterEnde = mktime(0,0,0,$month,1,$year);

    // Unixzeit vom Beginn des vorlaufenden Monats = Anfang des Zeitfensters
    $month--; if ($month <= 0) {$month=12; $year--;}
    $zeitfensterAnfang = mktime(0,0,0,$month,1,$year);

    return $this->zugriffeVonTimeBisTime($zeitfensterAnfang, $ZeitfensterEnde, $type);
  }

    // Liefert die Anzahl der Zugriffe, die lter als 2 Monate zurckliegen in den aktuellen LogLines
  function zugriffeAelterAlsZweiMonate()
  {
    // Unixzeit vom Beginn des vorlaufenden Monats = Anfang des Zeitfensters
    $month	= date("n",time());
    $year	= date("Y",time());
    $month--; if ($month <= 0) {$month=12; $year--;}
    $stichtagzeit = mktime(0,0,0,$month,1,$year);

    return $this->zugriffeVonTimeBisTime(0, $stichtagzeit, "ALL");
  }

  // Liefert ein Array mit den loglines, die lter als 2 Monate zurckliegen
  function loglinesAelterAlsZweiMonate()
  {
    // Unixzeit vom Beginn des vorlaufenden Monats = Anfang des Zeitfensters
    $month	= date("n",time());
    $year	= date("Y",time());
    $month--; if ($month <= 0) {$month=12; $year--;}
    $stichtagzeit = mktime(0,0,0,$month,1,$year);

    return $this->loglinesVonTimeBisTime(0, $stichtagzeit);
  }

  // Liefert ein Array mit den loglines, die jnger als 2 Monate zurckliegen
  function loglinesJuengerAlsZweiMonate()
  {
    // Unixzeit vom Beginn des vorlaufenden Monats = Anfang des Zeitfensters
    $month	= date("n",time());
    $year	= date("Y",time());
    $month--; if ($month <= 0) {$month=12; $year--;}
    $stichtagzeit = mktime(0,0,0,$month,1,$year);

    return $this->loglinesVonTimeBisTime($stichtagzeit,time());
  }

  // ----------------------------------------------------------------------------
  // -------------------- STATISCHE FUNKTIONEN ----------------------------------
  // ----------------------------------------------------------------------------

  // Aufruf ber LogFile::Funktionsname ohne zuvor ein Objekt zu instanzieren


  // Schreibt einen Logeintrag
  function writeLog($datei, $ip, $type)
  {
  // Fall der Dateiname einen Verzeichnispfad enthlt diesen zum Teil des Namens machen
  $id = str_replace("/","___v___",$datei);

  // Prfen, ob LogDatei angelegt werden muss
  if (!file_exists(LOGVERZEICHNIS.$id.".log"))
  {
   // LogDatei vorbereiten
   if ($fp = @fopen(LOGVERZEICHNIS.$id.".log","a"))
   {
     // Bereite Datei mit Init-Compressed-Zeile vor
     $oldLogs = array();
     $oldLogs['ALL'] = 0;
     $logline = serialize($oldLogs);
     @fwrite($fp,$logline."\r\n");
     @fclose($fp);
   }
  }



  if ($fp = @fopen(LOGVERZEICHNIS.$id.".log","a"))
  {
    // Datenarray des Logeintrages erstellen
    $log_array  = array();
    $log_array['UnixTime'] 	= time();
    $log_array['Date'] 		= date("d.m.Y H:i:s");
    $log_array['IP']		= $ip;
    $log_array['Type']		= $type;

    $log_line = serialize($log_array);

    @fwrite($fp,$log_line."\r\n");
    @fclose($fp);
  }
 }

 // gibt ein Array zurck, das die Namen aller Dateien enthlt zu den es logfile-Aufzeichnungen gibt
 function getAllLogFileNames()
 {
    $logArray = array();

    $verzeichnis = opendir(LOGVERZEICHNIS);
    while ($datei = readdir($verzeichnis))
    {
     $datNameTeile = explode(".",$datei);
     $endung = array_pop($datNameTeile);
     if (($endung == "log"))
     {
       array_push($logArray, str_replace("___v___","/",implode(".",$datNameTeile)));
     }
    }
    closedir($verzeichnis);

    return $logArray;
  }


  // Kann genutzt werden, um die Zugriffsdaten, die im vorletzten Monat liegen,
  // in der ersten Zeile der Logfiles verkrtzt zu speichern
  // Die Funktion komprimiert alle Logfiles, die es findet.

  function compressOldLogs()
  {
    // Alle Logs in Directory suchen
    $logDatnames = LogFile::getAllLogFileNames();

    foreach ($logDatnames as $logDatname)
    {
      $log = new LogFile($logDatname);

      $old_oldLogs = $log->getOldLogLineArray();

      if ($log->zugriffeAelterAlsZweiMonate() > 0)
      {
        print ("COMPRESS LOGGINGFILE: ".$logDatname."<br>");
        print (serialize($old_oldLogs)."<br>");
        // Es sind LogEintrge vorhanden, die in der ersten Zeile zusammen fasst werden knnen

        $new_actualLogs = $log->loglinesJuengerAlsZweiMonate();
        $old_actualLogs = $log->loglinesAelterAlsZweiMonate();

        // Alle alten actual-Loglines in die Kompromierte Zeile einzhlen
        foreach ($old_actualLogs as $old_logline)
        {
          // INSGESAMT erhhen
          $log->incOldLogLine();
          // TYP erhhen (wenn vorhanden)
          if (isset($old_logline['Type']))
          {
            $log->incOldLogLine($old_logline['Type']);
          }
        } // endforeach

        $new_oldLogs = $log->getOldLogLineArray();
        print (serialize($new_oldLogs)."<br>");

        // Alte LogDatei lschen und neue schreiben
        $convert_name =  str_replace("/","___v___",$logDatname.".log");
        @unlink(LOGVERZEICHNIS.$convert_name);

        if ($fp = fopen(LOGVERZEICHNIS.$convert_name,"a"))
        {
          // Komprimierte Zeile schreiben
          fwrite($fp,serialize($new_oldLogs)."\r\n");

          // die ausfhrlichen Loglines schreiben
          foreach ($new_actualLogs as $alog)
          {
            fwrite($fp,serialize($alog)."\r\n");
          }

          fclose($fp);
        }
      }
    }
  }

} // endclass

?>